import argparse, pickle, os
import torch.nn as nn
from torch import tensor, save
import torch.optim as optim
import numpy as np

from .main import MainLinear, MainDeep

def data_loader(batch=16):
	while True:
		inputs = np.random.uniform(size=[batch, 2])
		labels = np.zeros((batch, 2))
		idx = (np.linalg.norm(inputs, axis=1, ord=2) >= 1).astype(np.uint8)
		labels[np.arange(batch), idx] = 1
		yield tensor(inputs).float(), tensor(labels).float()

def accuracy(outputs, labels):
	outputs_idx = outputs.max(1)[1].type_as(outputs)
	labels_idx = labels.max(1)[1].type_as(labels)
	return outputs_idx.eq(labels_idx).float().mean()


def train_linear(model):
	'''
	Your code here
	'''

	log_step = 1000
	criterion = nn.BCEWithLogitsLoss()
	optimizer = optim.SGD(model.parameters(), lr=0.01, momentum=0.9)

	running_loss, running_acc = 0., 0.
	for i, (inputs, labels) in zip(range(10000), data_loader()):
		optimizer.zero_grad()

		# print (inputs, labels)

		outputs = model(inputs)
		loss = criterion(outputs, labels)
		acc = accuracy(outputs, labels)
		loss.backward()
		optimizer.step()

		running_loss += loss.item()
		running_acc += acc.item()
		if i % log_step == 0:
			print('step %d, loss:%.4f, acc: %.4f' % (i, running_loss/log_step, running_acc/log_step))
			running_loss = 0.
			running_acc = 0.

	dirname = os.path.dirname(os.path.abspath(__file__)) # Do NOT modify this line
	save(model.state_dict(), os.path.join(dirname, 'linear')) # Do NOT modify this line


def train_deep(model):
	'''
	Your code here
	'''

	log_step = 1000
	criterion = nn.BCEWithLogitsLoss()
	optimizer = optim.SGD(model.parameters(), lr=0.01, momentum=0.9)

	running_loss, running_acc = 0., 0.
	for i, (inputs, labels) in zip(range(10000), data_loader()):
		optimizer.zero_grad()

		# print (inputs, labels)

		outputs = model(inputs)
		loss = criterion(outputs, labels)
		acc = accuracy(outputs, labels)
		loss.backward()
		optimizer.step()

		running_loss += loss.item()
		running_acc += acc.item()
		if i % log_step == 0:
			print('step %d, loss:%.4f, acc: %.4f' % (i, running_loss/log_step, running_acc/log_step))
			running_loss = 0.
			running_acc = 0.

	dirname = os.path.dirname(os.path.abspath(__file__)) # Do NOT modify this line
	save(model.state_dict(), os.path.join(dirname, 'deep')) # Do NOT modify this line


if __name__ == '__main__':
	parser = argparse.ArgumentParser()
	parser.add_argument('model', choices=['linear', 'deep'])
	args = parser.parse_args()

	if args.model == 'linear':
		print ('[I] Start training linear model')
		train_linear(MainLinear())
	elif args.model == 'deep':
		print ('[I] Start training deep model')
		train_deep(MainDeep())

	print ('[I] Training finished')